import javax.management.AttributeList;
import javax.management.ObjectName;

import com.ibm.websphere.management.AdminClient;
import com.ibm.websphere.management.Session;
import com.ibm.websphere.management.configservice.ConfigService;
import com.ibm.websphere.management.configservice.ConfigServiceHelper;
import com.ibm.websphere.management.configservice.ConfigServiceProxy;
import com.ibm.websphere.management.exception.AdminException;


/**
 * Example 1 - Create a new application server
 */
public class Example1 {

    /**
     * Creates a new application server
     *
     * @param nodeName
     * @param serverName
     */
    public ObjectName createServer(String nodeName, String serverName) {

        ObjectName newServer = null;
        
        try {
            // Establish an AdminClient connected to a server.
            AdminClient adminClient = ExampleUtils.getAdminClient();

            // Create a config service proxy object.
            ConfigService configService = new ConfigServiceProxy(adminClient);

            // Create a config service session.
            Session session = new Session();

            // Query to get the config object for the node where we want to create
            // the new server.  The resolve method returns an array of ObjectNames
            // containing the config objects that match the query.  In this query
            // there will be one element if the node exists or none if the node
            // does not exist.
            ObjectName node = null;
            ObjectName[] nodes =
                configService.resolve(session, "Node=" + nodeName);
            if (nodes.length > 0)
                node = nodes[0];
            else
                throw new Exception("Node does not exist: " + nodeName);

            // Create the new application server.
            AttributeList attrList = new AttributeList();
            ConfigServiceHelper.setAttributeValue(attrList, "name", serverName);
            newServer = configService.createConfigData(
                session,
                node,             // the parent config object
                "Server",         // the relationship to the parent
                "Server",         // the type of the new config object
                attrList);

            // Save the new configuration (don't overwrite conflicting updates)
            configService.save(session, false);

            System.out.println("Created server " + serverName + " on node " + nodeName);

        }
        catch (Throwable t) {
            t.printStackTrace();
        }
        
        return newServer;
    }

    public static void main(String[] args) {
        
        // Create a new application server given the new server's name and the
        // name of the node on which to create it.
        if (args.length == 2) {
            Example1 ex1 = new Example1();
            ex1.createServer(args[0], args[1]);
        }
        else
            System.out.println("Usage: Example1 nodeName serverName");
        }
}
