import java.util.Hashtable;
import java.util.Locale;
import java.util.Set;

import javax.management.InstanceNotFoundException;
import javax.management.ListenerNotFoundException;
import javax.management.MalformedObjectNameException;
import javax.management.Notification;
import javax.management.NotificationFilterSupport;
import javax.management.NotificationListener;
import javax.management.ObjectName;

import com.ibm.websphere.management.AdminClient;
import com.ibm.websphere.management.application.AppConstants;
import com.ibm.websphere.management.application.AppManagement;
import com.ibm.websphere.management.application.AppManagementProxy;
import com.ibm.websphere.management.application.AppNotification;
import com.ibm.websphere.management.exception.ConnectorException;

/**
 * Example 2 - Install an application
 */
public class Example2 implements NotificationListener {

    AdminClient adminClient;
    ObjectName mBean;
    boolean complete = false;
    
    /**
     * Installs an application into the cell but does not target the
     * application to any servers.
     *
     * @param earFile  The path name of the ear file to install
     * @param appName  The name to give the installed application
     * @param cellName The name of the cell in which the application is being
     *                 installed
     */
    public void installApp(String earFile, String appName, String cellName) {

        try {
            // create AppManagement proxy to AppManagement MBean using
            // AdminClient
            adminClient = ExampleUtils.getAdminClient();
            AppManagement appMgmt = AppManagementProxy
                    .getJMXProxyForClient(adminClient);

            // get AppManagement MBean's ObjectName
            mBean = getAppManagementMBean(adminClient);

            // create event filter
            NotificationFilterSupport myFilter = new NotificationFilterSupport();
            myFilter.enableType(AppConstants.NotificationType);

            // add listener
            adminClient.addNotificationListener(mBean, this, myFilter, null);

            // prepare properties table
            Hashtable props = new Hashtable();
            props.put(AppConstants.APPDEPL_LOCALE, Locale.getDefault());
            props.put(AppConstants.APPDEPL_APPNAME, appName);
            props.put(AppConstants.APPDEPL_CELL, cellName);

            // call install
            appMgmt.installApplication(earFile, props, null);
            
            // wait for installation to complete
            waitForCompletion();
        } catch (Throwable t) {
            t.printStackTrace();
        }
    }

    // Gets the ObjectName of the AppManagement MBean
    private ObjectName getAppManagementMBean(AdminClient adminClient)
            throws Exception {
        
        ObjectName appMgmt = null;

        // Query for the ObjectName of the AppManagement MBean. Since there is
        // only one of these per cell all we need to specify in the query is
        // the type name

        String query = "WebSphere:type=AppManagement,*";
        ObjectName queryName = new ObjectName(query);
        Set s = adminClient.queryNames(queryName, null);

        // There should be exactly one AppManagement MBean
        if (!s.isEmpty())
            appMgmt = (ObjectName) s.iterator().next();
        else
            System.out.println("AppManagement MBean was not found");

        return appMgmt;
    }

    // listen to events
    public void handleNotification(Notification notf, Object handback) {
        AppNotification ev = (AppNotification) notf.getUserData();
        if (ev.taskName.equals(AppNotification.INSTALL)) {
            if (ev.taskStatus.equals(AppNotification.STATUS_COMPLETED)) {
                // install succeeded... remove listener
                System.out.println("Install succeeded");
                complete = true;
                try {
                    adminClient.removeNotificationListener(mBean, this);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else if (ev.taskStatus.equals(AppNotification.STATUS_FAILED)) {
                // install failed... remove listener
                System.out.println("Install failed.  See the server's SystemOut.log for details.");
                complete = true;
                try {
                    adminClient.removeNotificationListener(mBean, this);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                // install in progress
                System.out.println("Install in progress: " + ev.taskName);
            }
        }
    }

    private void waitForCompletion() throws Exception {
        while (!complete) {
            Thread.sleep(1000);
        }
    }
    
    public static void main(String[] args) {

        // Install a new application.
        if (args.length == 3) {
            Example2 ex2 = new Example2();
            ex2.installApp(args[0], args[1], args[2]);
        } else
            System.out.println("Usage: Example2 earFile appName cellName");
    }
}
