import java.util.Set;

import javax.management.ObjectName;

import com.ibm.websphere.management.AdminClient;

/**
 * Example4 - start an application server
 * 
 * This example uses the NodeAgent MBean's launchProcess operation to start
 * an application server.  Since this MBean resides in the nodeagent server
 * this example will only be useful in a Network Deployment environment.
 *
 */
public class Example4 {

    /**
     * Starts an application server.
     * 
     * @param nodeName    the name of the node on which the server resides
     * @param serverName  the name of the server to start
     */
    public void startServer(String nodeName, String serverName) {
        
        try {
            // Establish an AdminClient connected to a server.
            AdminClient adminClient = ExampleUtils.getAdminClient();

            // Get the node's NodeAgent MBean
            ObjectName nodeAgent = getNodeAgentMBean(adminClient, nodeName);

            // Invoke the NodeAgent's launchProcess operation in order to
            // start the server
            if (nodeAgent != null)
                launchServer(adminClient, nodeAgent, serverName);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
    }

    private ObjectName getNodeAgentMBean(
        AdminClient adminClient,
        String nodeName) {
        ObjectName appMgmt = null;

        // Query for the ObjectName of the NodeAgent MBean on the given node.
        // Type and node key properties will uniquely ID the node agent.
        try {
            String query = "WebSphere:*,type=NodeAgent,node=" + nodeName;
            ObjectName queryName = new ObjectName(query);
            Set s = adminClient.queryNames(queryName, null);

            // Since there is only one NodeAgent per node the result set will
            // either be empty if the node does not exist or the nodeagent
            // server is not running or have one element if the nodeagent
            // server is running.
            if (!s.isEmpty())
                appMgmt = (ObjectName) s.iterator().next();
            else
                System.out.println("NodeAgent MBean was not found");
        }
        catch (Throwable t) {
            t.printStackTrace();
        }

        return appMgmt;
    }

    private boolean launchServer(
        AdminClient adminClient,
        ObjectName nodeAgent,
        String serverName) {

        // Use the launchProcess operation on the NodeAgent MBean to start
        // the given server
        boolean launched = false;
        String opName = "launchProcess";
        String signature[] = { "java.lang.String" };
        String params[] = { serverName };
        try {
            Boolean b =
                (Boolean) adminClient.invoke(
                    nodeAgent,
                    opName,
                    params,
                    signature);
            launched = b.booleanValue();
            if (launched)
                System.out.println(serverName + " was launched");
            else
                System.out.println(serverName + " was not launched");

        }
        catch (Throwable t) {
            t.printStackTrace();
        }

        return launched;
    }

    public static void main(String[] args) {

        // Start an application server.
        if (args.length == 2) {
            Example4 ex4 = new Example4();
            ex4.startServer(args[0], args[1]);
        }
        else
            System.out.println("Usage: Example4 nodeName serverName");
    }
}
