import java.util.Set;

import javax.management.ObjectName;

import com.ibm.websphere.management.AdminClient;

/**
 * Example 5 - Stop an application server
 * 
 * This example uses the Server MBean's stop operation to stop the server.  In
 * a base, single server installation you would connect to the application server
 * you wish to stop.  In a Network Deployment installation you would connect to
 * the dmgr server and it will route the request to the application server.
 */
public class Example5 {

    /**
     * Stops an application server
     * 
     * @param nodeName    The name of the node on which the server resides
     * @param serverName  The name of the server to stop
     */
    public void stopServer(String nodeName, String serverName) {
        try {
            // Establish an AdminClient connected to a server.
            AdminClient adminClient = ExampleUtils.getAdminClient();

            // Get the server MBean
            ObjectName server =
                getServerMBean(adminClient, nodeName, serverName);

            // Invoke the server's stop operation
            if (server != null)
                stopServer(adminClient, server);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
    }

    private ObjectName getServerMBean(
        AdminClient adminClient,
        String nodeName,
        String serverName) {
        // Query for the ObjectName of the MBean of the server to be stopped.
        // To uniquely ID the MBean we'll need to specify the type, node, and
        // process name.
        ObjectName server = null;
        try {
            String query =
                "WebSphere:*,type=Server,node="
                    + nodeName
                    + ",process="
                    + serverName;
            ObjectName queryName = new ObjectName(query);
            Set s = adminClient.queryNames(queryName, null);

            // Since there is only one Server MBean per server the result set
            // will either be empty if the server does not exist or is not 
            // running or have one element if the server is running.
            if (!s.isEmpty())
                server = (ObjectName) s.iterator().next();
            else
                System.out.println("Server MBean was not found: " + query);
        }
        catch (Throwable t) {
            t.printStackTrace();
        }

        return server;
    }

    private void stopServer(AdminClient adminClient, ObjectName server) {
        // Use the stop operation on the Server MBean to stop
        // the given server
        String opName = "stop";
        String signature[] = null;
        String params[] = null;
        try {
            adminClient.invoke(server, opName, params, signature);
            System.out.println("The server was stopped");
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
    }

    public static void main(String[] args) {

        // Stop an application server.
        if (args.length == 2) {
            Example5 ex5 = new Example5();
            ex5.stopServer(args[0], args[1]);
        }
        else
            System.out.println("Usage: Example5 nodeName serverName");
    }
}
