import java.util.Hashtable;
import java.util.Locale;
import java.util.Set;

import javax.management.InstanceNotFoundException;
import javax.management.ListenerNotFoundException;
import javax.management.MalformedObjectNameException;
import javax.management.Notification;
import javax.management.NotificationFilterSupport;
import javax.management.NotificationListener;
import javax.management.ObjectName;

import com.ibm.websphere.management.AdminClient;
import com.ibm.websphere.management.application.AppConstants;
import com.ibm.websphere.management.application.AppManagement;
import com.ibm.websphere.management.application.AppManagementProxy;
import com.ibm.websphere.management.application.AppNotification;
import com.ibm.websphere.management.exception.ConnectorException;


/**
 * Example 8 - Unnstall an application
 */
public class Example8 implements NotificationListener {

    AdminClient adminClient;
    ObjectName mBean;
    boolean complete = false;
    
    /**
     * Example 8 - Uninstalls an application.
     *
     * @param appName  The name to give the installed application
     */
    public void uninstallApp(String appName) {

        try {
            // Create AppManagement proxy to AppManagement MBean using
            // AdminClient
            adminClient = ExampleUtils.getAdminClient();
            AppManagement appMgmt = AppManagementProxy
                    .getJMXProxyForClient(adminClient);

            // Get AppManagement MBean's ObjectName
            mBean = getAppManagementMBean(adminClient);

            // Create event filter
            NotificationFilterSupport myFilter = new NotificationFilterSupport();
            myFilter.enableType(AppConstants.NotificationType);

            // Add listener
            adminClient.addNotificationListener(mBean, this, myFilter, null);

            // Prepare properties table
            Hashtable props = new Hashtable();
            props.put(AppConstants.APPDEPL_LOCALE, Locale.getDefault());
            //props.put(AppConstants.APPDEPL_APPNAME, appName);
            //props.put(AppConstants.APPDEPL_CELL, cellName);

            // Call uninstall
            appMgmt.uninstallApplication(appName, props, null);
            
            // Wait for uninstall to complete;
            waitForCompletion();
        } catch (Throwable t) {
            t.printStackTrace();
        }
    }

    // Gets the ObjectName of the AppManagement MBean
    private ObjectName getAppManagementMBean(AdminClient adminClient)
            throws Exception {
        
        ObjectName appMgmt = null;

        // Query for the ObjectName of the AppManagement MBean. Since there is
        // only one of these per cell all we need to specify in the query is
        // the type name
        String query = "WebSphere:type=AppManagement,*";
        ObjectName queryName = new ObjectName(query);
        Set s = adminClient.queryNames(queryName, null);

        // There should be exactly one AppManagement MBean
        if (!s.isEmpty())
            appMgmt = (ObjectName) s.iterator().next();
        else
            System.out.println("AppManagement MBean was not found");

        return appMgmt;
    }

    // listen to events
    public void handleNotification(Notification notf, Object handback) {
        AppNotification ev = (AppNotification) notf.getUserData();
        if (ev.taskName.equals(AppNotification.UNINSTALL)) {
            if (ev.taskStatus.equals(AppNotification.STATUS_COMPLETED)) {
                // install succeeded... remove listener
                System.out.println("Uninstall succeeded");
                complete = true;
                try {
                    adminClient.removeNotificationListener(mBean, this);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else if (ev.taskStatus.equals(AppNotification.STATUS_FAILED)) {
                // install failed... remove listener
                System.out.println("Uninstall failed");
                complete = true;
                try {
                    adminClient.removeNotificationListener(mBean, this);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                // install in progress
                System.out.println("Uninstall in progress: " + ev.taskName);
            }
        }
    }

    private void waitForCompletion() throws Exception {
        while (!complete) {
            Thread.sleep(1000);
        }
    }

    public static void main(String[] args) {

        // Install a new application.
        if (args.length == 1) {
            Example8 ex8 = new Example8();
            ex8.uninstallApp(args[0]);
        } else
            System.out.println("Usage: Example8 appName");
    }
}
